/*
 * Copyright 2020 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

#ifndef CLOCKSTACK_H
#define CLOCKSTACK_H

/******** start customize part ********/

#if defined(__vms) && (defined(__alpha) || defined(__ia64))
#ifdef HIGHRES
#include <builtins.h>
#define TIMECOUNT_T unsigned long long int
#define GET_TIMECOUNT __RPCC()
#else
#include <starlet.h>

static inline long long int vms_get_timecount(void)
{
    long int res;
    long long int t;
    res = sys$gettim(&t);
    return (res & 1 == 1) ? t : 0;
}

#define TIMECOUNT_T long long int
#define GET_TIMECOUNT vms_get_timecount()
#endif
#endif

#ifdef __unix
#include <time.h>

static inline long long int unix_get_timecount(void)
{
    int res;
    struct timespec t;
    res = clock_gettime(CLOCK_MONOTONIC, &t);
    return (res == 0) ? (t.tv_sec * 1000000000LL + t.tv_nsec) : 0;
}

#define TIMECOUNT_T long long int
#define GET_TIMECOUNT unix_get_timecount()
#endif

#ifdef _WIN32
#ifdef HIGHRES
#include <windows.h>

#define TIMECOUNT_T long int
#define GET_TIMECOUNT GetTickCount()
#else
#include <windows.h>

static inline long long int win32_get_timecount(void)
{
    BOOL res;
    LARGE_INTEGER t;
    res = QueryPerformanceCounter(&t);
    return res ? t.QuadPart : 0;
}

#define TIMECOUNT_T long long int
#define GET_TIMECOUNT win32_get_timecount()
#endif
#endif


/******** end customize part ********/

#ifndef TIMECOUNT_T
#error "TIMECOUNT_T not defined"
#endif

#ifndef GET_TIMECOUNT
#error "GET_TIMECOUNT not defined"
#endif

#ifdef __cplusplus
extern "C"
{
#endif
void clockstack_enter(const char *func, TIMECOUNT_T timecount);
void clockstack_exit(const char *func, TIMECOUNT_T timecount);
void clockstack_dump(void);
#ifdef __cplusplus
}
#endif

#ifdef USE_CLOCKSTACK
#define FUNC_ENTER clockstack_enter(__func__, GET_TIMECOUNT);
#define FUNC_EXIT clockstack_exit(__func__, GET_TIMECOUNT);
#define DUMP_STATS clockstack_dump();
#else
#define FUNC_ENTER
#define FUNC_EXIT
#define DUMP_STATS
#endif

#endif
